%
%   [f,g] = sparsestressfun(x,DELTA,I)
%  
%   computes stress for config x given the high dimensional distances 
%   in the matrix DELTA 
%
%   we only consider the index pairs in I
%
%   x is a vectorized version of the points X where X stores the points as
%   rows = points and cols = dimensions
%
%   algorithm only works for two dimensions
%
%   f - stress value
%   g - gradient vector
%
function [f,g,H] = sparsestressfun(x,DELTA,I)

% unvectorize input
X(:,1)=x(1:size(DELTA,1));
X(:,2)=x(size(DELTA,1)+1:length(x));

% calculate stress
D=sparsefulldistance(X',I);
Itemp = unique([[I(:,1);I(:,2)] [I(:,2);I(:,1)]],'rows');

UNISEL = Itemp(:,1)+(Itemp(:,2)-1)*size(DELTA,1);

f = sum((D(UNISEL)-DELTA(UNISEL)).^2)/sum(DELTA(UNISEL).^2);

% f=(sum(arrayfun(@(x,y)(x<=y)*(D(x,y)-DELTA(x,y))^2,Itemp(:,1),Itemp(:,2))));
% f = f / sum(arrayfun(@(x,y)(x<y)*(DELTA(x,y))^2,Itemp(:,1),Itemp(:,2)));

%f = norm(triu(DELTA)-triu(D), 'fro')^2;

% compute gradient
if nargout > 1
    % construct V and B (from Borg)
    B = sparse(size(DELTA));
    V = sparse(size(DELTA));
    
    V = spfun(@(x)-1,D);
    V(1:size(V,1)+1:numel(V)) = -sum(V,2);
    B = sparse(Itemp(:,1),Itemp(:,2),-DELTA(UNISEL)./D(UNISEL));
%     B = sparse(Itemp(:,1),Itemp(:,2),arrayfun(@(x,y)-DELTA(x,y)/D(x,y),Itemp(:,1),Itemp(:,2)));
    B(1:size(B,1)+1:numel(B)) = 0;
    B(1:size(B,1)+1:numel(B)) = -sum(B,2);
    
    % compute gradient
    G = 2*V*X-2*B*X;
    g = G(:);
end

% compute Hessian
if nargout > 2
    n = length(x)/2;
    %H = sparse(length(x),length(x));
    Hcell = cell(2,2);
    %construct the hessian in NxN blocks
    for i = 1:2
        for j = i:2
            %Ht = sparse(n,n);
            Ht = sparse( [I(:,1);I(:,2)],[I(:,2);I(:,1)], arrayfun( @(a,b) -2*(x(a+(j-1)*n)-x(b+(j-1)*n))*(x(a+(i-1)*n)-x(b+(i-1)*n))*(a~=b)*DELTA(a,b)/(D(a,b)^3) , [I(:,1);I(:,2)],[I(:,2);I(:,1)] ), n, n );            
            %Ht = sparse( [I(:,1);I(:,2)],[I(:,2);I(:,1)], arrayfun( @(a,b) (a~=b)*(D(a,b)^3>=1e-5)*DELTA(a,b)/(D(a,b)^3) , [I(:,1);I(:,2)],[I(:,2);I(:,1)] ), n, n );            
            Ht(1:size(Ht,1)+1:numel(Ht)) = -sum(Ht,2);
            Hcell{i,j}=Ht;
%             for k = 1:size(I,1)
%                 if I(k,1) ~= I(k,2)
%                     if D(I(k,1),I(k,2)) < 1e-6
%                         Ht(I(k,1),I(k,2)) = 0;
%                     else
%                         a = x(I(k,1)+(i-1)*n)-x(I(k,2)+(i-1)*n);
%                         b = x(I(k,1)+(j-1)*n)-x(I(k,2)+(j-1)*n);
%                         Ht(I(k,1),I(k,2)) = -2*a*b*DELTA(I(k,1),I(k,2))/(D(I(k,1),I(k,2))^3);
%                     end
%                     if D(I(k,2),I(k,1)) < 1e-6
%                         Ht(I(k,2),I(k,1)) = 0;
%                     else
%                         a = x(I(k,2)+(i-1)*n)-x(I(k,1)+(i-1)*n);
%                         b = x(I(k,2)+(j-1)*n)-x(I(k,1)+(j-1)*n);
%                         Ht(I(k,2),I(k,1)) = -2*a*b*DELTA(I(k,2),I(k,1))/(D(I(k,2),I(k,1))^3);
%                     end
%                 end
%             end
%             for k = 1:size(D,1)
%               Ht(k,k) = -sum(Ht(k,:));
%             end
            %H(((i-1)*n+1):(i*n),((j-1)*n+1):(j*n))=Ht;
            if i ~= j
                %H(((j-1)*n+1):(j*n),((i-1)*n+1):(i*n))=Ht';
            end
        end
    end
    H=[Hcell{1,1},Hcell{1,2};Hcell{1,2}' Hcell{2,2}];
end