%
%   [f,g] = stressfun(x,DELTA)
%  
%   computes stress for config x given the high dimensional distances 
%   in the matrix DELTA 
%
%   x is a vectorized version of the points X where X stores the points as
%   rows = points and cols = dimensions
%
%   algorithm only works for two dimensions
%
%   f - stress value
%   g - gradient vector
%
function [f,g] = stressfun(x,DELTA,W)

% unvectorize input
X(:,1)=x(1:size(DELTA,1));
X(:,2)=x(size(DELTA,1)+1:length(x));

% calculate stress
D=fulldistance(X');
%f = norm(triu(DELTA)-triu(D), 'fro')^2;
f = (sum(sum((triu(DELTA)-triu(D)).^2)))/sum(sum(triu(DELTA).^2));

% compute gradient
if nargout > 1
    % construct V and B (from Borg)
    B = zeros(size(DELTA));
    V = zeros(size(DELTA));
    if nargin > 2
        V = -W;
        V(1:size(V,1)+1:numel(V)) = sum(W,2);
        B = (-W.*DELTA)./D;
        B(1:size(B,1)+1:numel(B)) = 0;
        B(1:size(B,1)+1:numel(B)) = -sum(B,2);
    else
        V = -ones(size(D));
        V(1:size(V,1)+1:numel(V)) = -sum(V,2);
        B = -DELTA./D;
        B(1:size(B,1)+1:numel(B)) = 0;
        B(1:size(B,1)+1:numel(B)) = -sum(B,2);
    end
%     for j = 1:size(D,1)
%       B(j,j) = 0;
%       for k = 1:size(D,2)
%         if( j ~= k )
%           if nargin > 2
%             V(j,k)=-W(j,k);
%           else
%             V(j,k)=-1;
%           end
%           if D(j,k) == 0 
%             B(j,k) = 0;
%           else
%             if nargin > 2
%                 B(j,k) = -W(j,k)*DELTA(j,k)/D(j,k);
%             else
%                 B(j,k) = -DELTA(j,k)/D(j,k);
%             end
%           end
%         end
%       end
%     end
%     for j = 1:size(D,1)
%       B(j,j) = -sum(B(j,:));
%       V(j,j) = -sum(V(j,:));
%     end

    % compute gradient
    G = 2*V*X-2*B*X;
    g = G(:);
end

% compute Hessian
if nargout > 2

end